import typing
from collections import defaultdict
import json

import yaml
from yaml.scanner import ScannerError

import apiman


STATIC_DIR = f"{getattr(apiman, '__path__')[0]}/static/"


class OpenApi:
    SPECIFICATION_FILE = "__spec_file__"
    CONFIG = {
        "title": "OpenAPI Document",
        "specification_url": "/apiman/schema/",
        "swagger_url": "/apiman/swagger/",
        "redoc_url": "/apiman/redoc/",
        "swagger_template": f"{STATIC_DIR}swagger.html",
        "redoc_template": f"{STATIC_DIR}redoc.html",
    }

    def __init__(self, template=f"{STATIC_DIR}template.yml", **config):
        self.specification = self.load_file(template)
        self.config = self.CONFIG.copy()
        self.config.update(config)
        self.loaded = False

        if self.is_swagger:
            self.specification.setdefault("definitions", {})
        else:
            self.specification.setdefault("components", {"schemas": {}})
        paths = self.specification.pop("paths", {})
        self.specification["paths"] = defaultdict(dict)
        for k, v in paths.items():
            self.specification["paths"][k] = v

    @property
    def is_swagger(self):
        return "swagger" in self.specification

    def add_schema(self, name: str, definition: typing.Dict):
        if self.is_swagger:
            self.specification["definitions"][name] = definition
        else:
            self.specification["components"]["schemas"][name] = definition

    def from_file(self, file_path: str) -> typing.Callable:
        def decorator(func: typing.Callable) -> typing.Callable:
            setattr(func, self.SPECIFICATION_FILE, file_path)
            return func

        return decorator

    def from_doc(self, func_or_method: typing.Callable) -> typing.Dict:
        if not func_or_method.__doc__:
            return {}

        try:
            spec = yaml.safe_load(func_or_method.__doc__.split("---")[-1])
        except ScannerError:
            return {}
        if not isinstance(spec, dict):
            return {}

        return spec

    def _load_specification(self, path: str, method: str, specification: typing.Dict):
        schemas = specification.pop("definitions" if self.is_swagger else "schemas", {})
        for name, schema in schemas.items():
            self.add_schema(name, schema)

        self.specification["paths"][path][method.lower()] = specification

    @staticmethod
    def load_file(file_path: str) -> typing.Dict:
        with open(file_path) as f:
            if file_path.endswith("json"):
                return json.load(f)
            else:
                return yaml.safe_load(f)



import typing

from starlette.applications import Starlette
from starlette.responses import Response, JSONResponse
from starlette.schemas import SchemaGenerator
from jinja2 import Template

from .openapi import OpenApi


class Extension(OpenApi):
    def init_app(self, app: Starlette):
        if self.config["swagger_template"] and self.config["swagger_url"]:
            swagger_html = Template(
                open(self.config["swagger_template"]).read()
            ).render(self.config)
            app.add_route(
                self.config["swagger_url"],
                lambda _: Response(swagger_html),
                methods=["GET"],
                include_in_schema=False,
            )
        if self.config["redoc_template"] and self.config["redoc_url"]:
            redoc_html = Template(open(self.config["redoc_template"]).read()).render(
                self.config
            )
            app.add_route(
                self.config["redoc_url"],
                lambda _: Response(redoc_html),
                methods=["GET"],
                include_in_schema=False,
            )
        if self.config["specification_url"]:
            app.add_route(
                self.config["specification_url"],
                lambda _: JSONResponse(self.load_specification(app)),
                methods=["GET"],
                include_in_schema=False,
            )

    def load_specification(self, app: Starlette) -> typing.Dict:
        if app.debug or not self.loaded:
            for endpoint in SchemaGenerator({}).get_endpoints(app.routes):
                specification = self.from_doc(endpoint.func)
                if not specification and hasattr(
                    endpoint.func, self.SPECIFICATION_FILE
                ):
                    specification = self.load_file(
                        getattr(endpoint.func, self.SPECIFICATION_FILE)
                    )
                if specification:
                    self._load_specification(
                        endpoint.path, endpoint.http_method, specification
                    )
            self.loaded = True
            return self.specification
        else:
            return self.specification



#!/usr/bin/env python3
from setuptools import setup, find_packages
import re

with open("apiman/__init__.py", "r", encoding="utf8") as f:
    version = re.search(r"__version__ = \"(.*?)\"", f.read()).group(1)

requires = ["PyYAML>=3.13", "Jinja2>=2.10"]
tests_requires = [
    "pytest>=3.3.1",
    "pytest-cov>=2.5.1",
    "openapi_spec_validator==0.2.5",
    "jsonschema<3",
    "uvicorn>=0.3.24",
    "starlette>=0.11.1",
    "flask>=1.0",
]
setup_requires = ["pytest-runner>=3.0"]

setup(
    name="apiman",
    version=version,
    url="https://github.com/strongbugman/apiman",
    description="Integrate api manual for your web project",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Bruce Wu",
    author_email="strongbugman@gmail.com",
    license="BSD",
    classifiers=[
        "Environment :: Console",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(exclude=("tests", "tests.*", "examples")),
    install_requires=requires,
    setup_requires=setup_requires,
    tests_require=tests_requires,
    include_package_data=True,
)



import typing

from flask import Flask, jsonify, Response
from jinja2 import Template

from .openapi import OpenApi


class Extension(OpenApi):
    def init_app(self, app: Flask):
        self.config.update(app.config.get_namespace("OPNEAPI_", lowercase=True))

        if self.config["swagger_template"] and self.config["swagger_url"]:
            swagger_html = Template(
                open(self.config["swagger_template"]).read()
            ).render(self.config)
            app.route(
                self.config["swagger_url"], methods=["GET"], endpoint="swagger_ui"
            )(lambda: Response(swagger_html))
        if self.config["redoc_template"] and self.config["redoc_url"]:
            redoc_html = Template(open(self.config["redoc_template"]).read()).render(
                self.config
            )
            app.route(self.config["redoc_url"], methods=["GET"], endpoint="redoc_ui")(
                lambda: Response(redoc_html)
            )
        if self.config["specification_url"]:
            app.route(
                self.config["specification_url"], endpoint="openapi_specification"
            )(lambda: jsonify(self.load_specification(app)))

    def load_specification(self, app: Flask) -> typing.Dict:
        if app.debug or not self.loaded:
            for route in app.url_map.iter_rules():
                func = app.view_functions[route.endpoint]
                for method in route.methods:
                    _func = getattr(func, method, func)
                    specification = self.from_doc(_func)
                    if not specification and hasattr(_func, self.SPECIFICATION_FILE):
                        specification = self.load_file(
                            getattr(_func, self.SPECIFICATION_FILE)
                        )
                    if specification:
                        self._load_specification(route.rule, method, specification)
            self.loaded = True
            return self.specification
        else:
            return self.specification
